/**
 * Copyright © 2014-2025 PDF Technologies, Inc. All Rights Reserved.
 *
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */
import { CPDFWidgetType } from "../../configuration/CPDFOptions";
import { CPDFCheckboxWidget } from "./CPDFCheckboxWidget";
import { CPDFComboboxWidget } from "./CPDFComboboxWidget";
import { CPDFListboxWidget } from "./CPDFListboxWidget";
import { CPDFPushbuttonWidget } from "./CPDFPushbuttonWidget";
import { CPDFRadiobuttonWidget } from "./CPDFRadiobuttonWidget";
import { CPDFSignatureWidget } from "./CPDFSignatureWidget";
import { CPDFTextWidget } from "./CPDFTextWidget";
import { CPDFWidget } from "./CPDFWidget";

export class CPDFWidgetFactory {
    
    private static widgetMap = new Map<CPDFWidgetType, (json: any) => CPDFWidget>([
        [CPDFWidgetType.TEXT_FIELD, CPDFTextWidget.fromJson.bind(CPDFTextWidget)],
        [CPDFWidgetType.SIGNATURES_FIELDS, CPDFSignatureWidget.fromJson.bind(CPDFSignatureWidget)],
        [CPDFWidgetType.RADIO_BUTTON, CPDFRadiobuttonWidget.fromJson.bind(CPDFRadiobuttonWidget)],
        [CPDFWidgetType.PUSH_BUTTON, CPDFPushbuttonWidget.fromJson.bind(CPDFPushbuttonWidget)],
        [CPDFWidgetType.LISTBOX, CPDFListboxWidget.fromJson.bind(CPDFListboxWidget)],
        [CPDFWidgetType.COMBOBOX, CPDFComboboxWidget.fromJson.bind(CPDFComboboxWidget)],
        [CPDFWidgetType.CHECKBOX, CPDFCheckboxWidget.fromJson.bind(CPDFCheckboxWidget)]
    ]);

    static create(json: any): CPDFWidget {
        const type = CPDFWidget.parseType(json.type);
        const widgetCreator = CPDFWidgetFactory.widgetMap.get(type);
        return widgetCreator ? widgetCreator(json) : CPDFWidget.fromJson(json);
    }

    static createFromArray(jsonArray: any[]): CPDFWidget[] {
        return jsonArray.map(CPDFWidgetFactory.create);
    }
}